<!DOCTYPE html>

<html>

<head>

<script src="http://code.jquery.com/jquery-1.9.1.min.js"></script>

<script>

$(document).ready(function() {
  $('#closer').click(function(e) {
    e.preventDefault();
    window.close();
  });
});

</script>

</head>

<body>

<h1>Yay!</h1>

<ul>
  {% for key, value in data.items %}
    <li>{{ key }}&nbsp;&nbsp;{{ value }}</li>
  {% endfor %}
</ul>

<a href="#" id="closer">Close me</a>

</body>

</html>
