<!DOCTYPE html>

<html>

<head>
</head>

<body>

{% for form in forms %}

<!-- BuySafe -->
<form name="buysafe" method="POST" action="https://test.esafe.com.tw/Service/Etopm.aspx">
  {{ form }}
  <button type="submit">{{ form.submit_button_title }}</button>
</form>

{% endfor %}

</body>

</html>
