<!DOCTYPE html>

<html>

<head>

<script src="http://code.jquery.com/jquery-1.9.1.min.js"></script>

<script>
$(document).ready(function() {
  $('#start_form').submit(function(e) {
    window.open('', 'popup', 'width=600,height=750');
  });
});
</script>

</head>

<body>

<form id="start_form" method="POST" action="{% url 'buysafe.views.start' %}" target="popup">
  {% csrf_token %}
  <input type="hidden" name="order_id" value="{{ order_id }}" />
  <button type="submit">Submit</button>
</form>

</body>

</html>
