<!DOCTYPE html>

<html>

<head>
</head>

<body>

<h1>Ooops.</h1>

<ul>
  {% for key, value in data.items %}
    <li>{{ key }}&nbsp;&nbsp;{{ value }}</li>
  {% endfor %}
</ul>

<a href="{% url 'buysafe.views.start' %}">Do it again</a>

</body>

</html>
